/***************************************************************************//**
* \file UART2_PM.c
* \version 4.0
*
* \brief
*  This file provides the source code to the Power Management support for
*  the SCB Component.
*
* Note:
*
********************************************************************************
* \copyright
* Copyright 2013-2017, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "UART2.h"
#include "UART2_PVT.h"

#if(UART2_SCB_MODE_I2C_INC)
    #include "UART2_I2C_PVT.h"
#endif /* (UART2_SCB_MODE_I2C_INC) */

#if(UART2_SCB_MODE_EZI2C_INC)
    #include "UART2_EZI2C_PVT.h"
#endif /* (UART2_SCB_MODE_EZI2C_INC) */

#if(UART2_SCB_MODE_SPI_INC || UART2_SCB_MODE_UART_INC)
    #include "UART2_SPI_UART_PVT.h"
#endif /* (UART2_SCB_MODE_SPI_INC || UART2_SCB_MODE_UART_INC) */


/***************************************
*   Backup Structure declaration
***************************************/

#if(UART2_SCB_MODE_UNCONFIG_CONST_CFG || \
   (UART2_SCB_MODE_I2C_CONST_CFG   && (!UART2_I2C_WAKE_ENABLE_CONST))   || \
   (UART2_SCB_MODE_EZI2C_CONST_CFG && (!UART2_EZI2C_WAKE_ENABLE_CONST)) || \
   (UART2_SCB_MODE_SPI_CONST_CFG   && (!UART2_SPI_WAKE_ENABLE_CONST))   || \
   (UART2_SCB_MODE_UART_CONST_CFG  && (!UART2_UART_WAKE_ENABLE_CONST)))

    UART2_BACKUP_STRUCT UART2_backup =
    {
        0u, /* enableState */
    };
#endif


/*******************************************************************************
* Function Name: UART2_Sleep
****************************************************************************//**
*
*  Prepares the UART2 component to enter Deep Sleep.
*  The “Enable wakeup from Deep Sleep Mode” selection has an influence on this 
*  function implementation:
*  - Checked: configures the component to be wakeup source from Deep Sleep.
*  - Unchecked: stores the current component state (enabled or disabled) and 
*    disables the component. See SCB_Stop() function for details about component 
*    disabling.
*
*  Call the UART2_Sleep() function before calling the 
*  CyPmSysDeepSleep() function. 
*  Refer to the PSoC Creator System Reference Guide for more information about 
*  power management functions and Low power section of this document for the 
*  selected mode.
*
*  This function should not be called before entering Sleep.
*
*******************************************************************************/
void UART2_Sleep(void)
{
#if(UART2_SCB_MODE_UNCONFIG_CONST_CFG)

    if(UART2_SCB_WAKE_ENABLE_CHECK)
    {
        if(UART2_SCB_MODE_I2C_RUNTM_CFG)
        {
            UART2_I2CSaveConfig();
        }
        else if(UART2_SCB_MODE_EZI2C_RUNTM_CFG)
        {
            UART2_EzI2CSaveConfig();
        }
    #if(!UART2_CY_SCBIP_V1)
        else if(UART2_SCB_MODE_SPI_RUNTM_CFG)
        {
            UART2_SpiSaveConfig();
        }
        else if(UART2_SCB_MODE_UART_RUNTM_CFG)
        {
            UART2_UartSaveConfig();
        }
    #endif /* (!UART2_CY_SCBIP_V1) */
        else
        {
            /* Unknown mode */
        }
    }
    else
    {
        UART2_backup.enableState = (uint8) UART2_GET_CTRL_ENABLED;

        if(0u != UART2_backup.enableState)
        {
            UART2_Stop();
        }
    }

#else

    #if (UART2_SCB_MODE_I2C_CONST_CFG && UART2_I2C_WAKE_ENABLE_CONST)
        UART2_I2CSaveConfig();

    #elif (UART2_SCB_MODE_EZI2C_CONST_CFG && UART2_EZI2C_WAKE_ENABLE_CONST)
        UART2_EzI2CSaveConfig();

    #elif (UART2_SCB_MODE_SPI_CONST_CFG && UART2_SPI_WAKE_ENABLE_CONST)
        UART2_SpiSaveConfig();

    #elif (UART2_SCB_MODE_UART_CONST_CFG && UART2_UART_WAKE_ENABLE_CONST)
        UART2_UartSaveConfig();

    #else

        UART2_backup.enableState = (uint8) UART2_GET_CTRL_ENABLED;

        if(0u != UART2_backup.enableState)
        {
            UART2_Stop();
        }

    #endif /* defined (UART2_SCB_MODE_I2C_CONST_CFG) && (UART2_I2C_WAKE_ENABLE_CONST) */

#endif /* (UART2_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: UART2_Wakeup
****************************************************************************//**
*
*  Prepares the UART2 component for Active mode operation after 
*  Deep Sleep.
*  The “Enable wakeup from Deep Sleep Mode” selection has influence on this 
*  function implementation:
*  - Checked: restores the component Active mode configuration.
*  - Unchecked: enables the component if it was enabled before enter Deep Sleep.
*
*  This function should not be called after exiting Sleep.
*
*  \sideeffect
*   Calling the UART2_Wakeup() function without first calling the 
*   UART2_Sleep() function may produce unexpected behavior.
*
*******************************************************************************/
void UART2_Wakeup(void)
{
#if(UART2_SCB_MODE_UNCONFIG_CONST_CFG)

    if(UART2_SCB_WAKE_ENABLE_CHECK)
    {
        if(UART2_SCB_MODE_I2C_RUNTM_CFG)
        {
            UART2_I2CRestoreConfig();
        }
        else if(UART2_SCB_MODE_EZI2C_RUNTM_CFG)
        {
            UART2_EzI2CRestoreConfig();
        }
    #if(!UART2_CY_SCBIP_V1)
        else if(UART2_SCB_MODE_SPI_RUNTM_CFG)
        {
            UART2_SpiRestoreConfig();
        }
        else if(UART2_SCB_MODE_UART_RUNTM_CFG)
        {
            UART2_UartRestoreConfig();
        }
    #endif /* (!UART2_CY_SCBIP_V1) */
        else
        {
            /* Unknown mode */
        }
    }
    else
    {
        if(0u != UART2_backup.enableState)
        {
            UART2_Enable();
        }
    }

#else

    #if (UART2_SCB_MODE_I2C_CONST_CFG  && UART2_I2C_WAKE_ENABLE_CONST)
        UART2_I2CRestoreConfig();

    #elif (UART2_SCB_MODE_EZI2C_CONST_CFG && UART2_EZI2C_WAKE_ENABLE_CONST)
        UART2_EzI2CRestoreConfig();

    #elif (UART2_SCB_MODE_SPI_CONST_CFG && UART2_SPI_WAKE_ENABLE_CONST)
        UART2_SpiRestoreConfig();

    #elif (UART2_SCB_MODE_UART_CONST_CFG && UART2_UART_WAKE_ENABLE_CONST)
        UART2_UartRestoreConfig();

    #else

        if(0u != UART2_backup.enableState)
        {
            UART2_Enable();
        }

    #endif /* (UART2_I2C_WAKE_ENABLE_CONST) */

#endif /* (UART2_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/* [] END OF FILE */
