/*******************************************************************************
* File Name: main.c
*
* Version: 1.0
*
* Description:
*  This project demonstrates simultaneous usage of the BLE GAP Peripheral and
*  Broadcaster roles. The device would connect to a peer device, while also
*  broadcasting (non-connectable advertising) at the same time.
*
* Hardware Dependency:
*  CY8CKIT-042-BLE Bluetooth Low Energy Pioneer Kit
*

=========================================================================================================================================================
Cypress External Response:39/13/2019 9:50 AM - P Yugandhar
Hello Luigi,

Sorry for the delayed response.

1. Please refer to the attached project.
   In the project, I configured the BLE as both central and peripheral and in the stack on event I'm starting the advertisement and scanning at the same time.
   In the event CYBLE_EVT_GAPC_SCAN_PROGRESS_RESULT, we can view all the nearby advertising devices.

2. You can add a switch button for switching the role of the device(either to advertising or scanning) on every switch interrupt.
   You can check the current state of the device using the CyBle_GetState() API function and can switch accordingly.

3. The project is configured with low power solution in which between every advertising or connection interval the device enters into the Deepsleep mode
   and the BLESS interrupt will wakes up the system from DeepSleep before start of every connection or advertisement event.
   Please refer to the below Datasheet link for more information on low power design of PSoC 4/PRoC BLE.
   Link: https://www.cypress.com/documentation/application-notes/an92584-designing-low-power-and-estimating-battery-life-ble

Please let me know if you have any questions.

Thanks,
P Yugandhar.
=========================================================================================================================================================
*/

/*
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

GG: Il modulo funziona in 2 modalità:  Advertiser e Observer (Broadcast receiver)
    
    RX da UART:
    La caratteristica sulla quale il CLIENT scrive è "BA6BD927-6363-4EDF-B0D6-189B0B53824F"   handle 0x17

    TX su BLE:
    Le Notifiche si abilitano scrivendo 0x01 alla caratteristica "2902"  handle 0x14  (descriptor:  "60E66627-EA26-4D9B-8142-42E30F24074D")
    La caratteristica sulla quale il modulo BLE invia Notifiche è "8D738CA0-FFED-47A4-8448-9D34B2D946E3"  handle 0x12

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
*/
/*******************************************************************************
* Included headers
*******************************************************************************/

#define __MAIN_H

#include <project.h>
#include <stdlib.h>
#include <stdio.h>
#include <typedef.h>
#include <common.h>
#include <bufcir.h>
#include <timer.h>
#include <rout.h>
#include <UART.h>
#include <uart2Drv.h>
#include <main.h>


/***************************************
*        Function Prototypes
***************************************/
void StackEventHandler(uint32 event, void* eventParam);

extern CY_ISR(U2Irq);
extern CY_ISR(Timer2mS5);
extern CY_ISR(UART2_SCB_IRQ_Interrupt);

//================================================================================================
static void BleRxHandlerInit(void)
//================================================================================================
{
    MemFill(U08PTR comm, 0, sizeof(comm));
  	BufCirInit((STRUCT_BUFCIR *)&comm.buffCircRx, U08PTR comm.buffRx,  RXBUF_LEN);
}

//=====================================================================
CYBLE_API_RESULT_T RefreshTxCharacteristic(u08 *p, u08 len)
//=====================================================================
{
	CYBLE_GATTS_WRITE_REQ_PARAM_T       wrReqParam;
    CYBLE_API_RESULT_T                  errore;

	wrReqParam.handleValPair.attrHandle = BLE_GATT_NOTIFY_HANDLE;
	wrReqParam.handleValPair.value.len = len;
	wrReqParam.handleValPair.value.val = p;
    /* Report data to BLE component for sending data by notifications*/
    errore = CyBle_GattsNotification(cyBle_connHandle, &wrReqParam.handleValPair);
    return errore;
}        


//==================================================================================================
void StackEventHandler(uint32 event, void* eventParam)
//==================================================================================================
{
   CYBLE_GAPC_ADV_REPORT_T advReport;
	CYBLE_GATTS_WRITE_REQ_PARAM_T       *wrReqParam;
	CYBLE_GATTS_CHAR_VAL_READ_REQ_T     *rdReqParam;
    CYBLE_GATT_ERR_CODE_T               errore;
	char answerRequest =0;

    switch(event)
    {

        case CYBLE_EVT_STACK_ON:    /*BLE stack ON*/

                printf("\r\n-------------------------------\r\n\r\n");
                printf("BLE_ON\r\n");
                //
                // In the project, I configured the BLE as both central and peripheral.
                //
                // In the stack on event (here) I'm starting the advertisement and scanning at the same time.
                // In the event CYBLE_EVT_GAPC_SCAN_PROGRESS_RESULT, we can view all the nearby advertising devices

                // You can add a switch button for switching the role of the device(either to advertising or scanning) on every switch interrupt.
                // You can check the current state of the device using the CyBle_GetState() API function and can switch accordingly.
                //
                // The project is configured with low power solution in which between every advertising or connection interval the device enters into the Deepsleep mode
                // and the BLESS interrupt will wake up the system from DeepSleep before start of every connection or advertisement event.
                // Please refer to the below Datasheet link for more information on low power design of PSoC 4/PRoC BLE.
                // Link: https://www.cypress.com/documentation/application-notes/an92584-designing-low-power-and-estimating-battery-life-ble
                //

/*
        LINK:    https://www.silabs.com/community/wireless/bluetooth/knowledge-base.entry.html/2018/05/30/ble_basics_masters-i4n9

        BLE Quick Overview

        >> Master (or "central") devices scan for other devices. Usually, the master is the smartphone/tablet/PC.
        >> Slave (or "peripheral") devices advertise and wait for connections. Usually, the slave is the BGMxxx module of EFR32 device.
        >> Client devices access remote resources over a BLE link using the GATT protocol. Usually, the master is also the client.
        >> Server devices have a local database and access control methods, and provide resources to the remote client. Usually, the slave is also the server.

        You can use read, write, notify, or indicate operations to move data between the client and the server.
        Read and write operations are requested by the client and the server responds (or acknowledges).
        Notify and indicate operations are enabled by the client but initiated by the server, providing a way to push data to the client.
        Notifications are unacknowledged, while indications are acknowledged. Notifications are therefore faster, but less reliable.

        Master vs. Slave - Connection Roles
        One important concept in BLE connectivity is the difference between a master device and a slave device.
        What do each of these terms imply?
        First, realize that they are not interchangeable with client/server, which will be explained below.
        Actually, in the BLE world, the master/slave difference is very easy to define and recognize:
            Master / Central - the BLE device which initiates an outgoing connection request to an advertising peripheral device
            Slave / Peripheral - the BLE device which accepts an incoming connection request after advertising

*/
                cyBle_discoveryInfo.discProcedure=CYBLE_GAPC_GEN_DISC_PROCEDURE;  //_GG_
                CyBle_GappStartAdvertisement(CYBLE_ADVERTISING_FAST);
                CyBle_GapcStartDiscovery(&cyBle_discoveryInfo); //_GG_
            break;

        case CYBLE_EVT_STACK_BUSY_STATUS:
            printf("(STACK_BUSY)\n");
            break;

        case CYBLE_EVT_TIMEOUT: /* 0x01 -> GAP limited discoverable mode timeout. */
                                /* 0x02 -> GAP pairing process timeout. */
                                /* 0x03 -> GATT response timeout. */
            printf("(EVT_TOUT: %d)\n", *(uint8 *)eventParam);
            break;

        case CYBLE_EVT_HARDWARE_ERROR:    /* This event indicates that some internal HW error has occurred. */
            printf("(HARDWARE_ERROR)\n");
            break;

        case CYBLE_EVT_HCI_STATUS:
            printf("(HCI_STATUS)\n");
            break;

        case CYBLE_EVT_GAPC_SCAN_PROGRESS_RESULT:

                /* scan progress result event occurs
                 * when it receives any advertisiment packet
                 * or scan response packet from peer device*/

                advReport=*(CYBLE_GAPC_ADV_REPORT_T *)eventParam;

                switch(advReport.eventType)  // (_GG_) enum CYBLE_GAPC_ADV_EVENT_T
                {
                    case CYBLE_GAPC_CONN_UNDIRECTED_ADV:
                        //_GG_ printf("Connectable undirected advertising\r\n");
                        break;
                    case  CYBLE_GAPC_CONN_DIRECTED_ADV:
                        //_GG_ printf("Connectable directed advertising\r\n");
                        break;
                    case  CYBLE_GAPC_SCAN_UNDIRECTED_ADV:
                        //_GG_ printf("Scannable undirected advertising\r\n");
                        break;

                    case CYBLE_GAPC_NON_CONN_UNDIRECTED_ADV:
                        //
                        // Broadcast Packed Received: quello che mi interessa per Rx dati SENSORI BLE
                        //
                        if(advReport.peerAddrType==CYBLE_GAP_ADDR_TYPE_PUBLIC)
                        {
#ifdef PRINT_SU_UART1                            
                            /*  NCU: NonConnectableUndirect */
                                printf("%05d NCU:%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X\r\n",
                                millisecCtr,
                                advReport.peerBdAddr[5u], advReport.peerBdAddr[4u],
                                advReport.peerBdAddr[3u], advReport.peerBdAddr[2u],
                                advReport.peerBdAddr[1u], advReport.peerBdAddr[0u]);


                            //  RSSI of the received packet from peer Device
                            printf("RSSI:%04d\r\n", advReport.rssi);
                            */
#endif                            
                            if (!memcmp(advReport.data,"\x2\x1\x6\x9\x8iSensor ",13))
                            {
                                // identificativo Sensore Ok
                                char tmp[50];
                                //char tmp2[50];
                                memset(tmp,0,sizeof(tmp));
                                
                                //memset(tmp2,0,sizeof(tmp2));
                                //memcpy(tmp2,&advReport.data[13],10);

                                sprintf(tmp, "&&&mS:%05d,RSSI:%03d,BCST:%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X\r\n",
                                millisecCtr,
                                advReport.rssi,
                                advReport.data[13u], advReport.data[14u],
                                advReport.data[15u], advReport.data[16u],
                                advReport.data[17u], advReport.data[18u],
                                advReport.data[19u], advReport.data[20u],
                                advReport.data[21u], advReport.data[22u]);
                                Uart2SendString(tmp);
                            }
                        }
                        break;

                    case CYBLE_GAPC_SCAN_RSP:
                        //
                        //  _GG_ BLE_GAPC_SCAN_RSP Scan Response
                        //
                        // printf("SCAN_RSP\r\n");
                        break;
                }
            break;

         case  CYBLE_EVT_GAPC_SCAN_START_STOP:
                if(CyBle_GetState()==CYBLE_STATE_DISCONNECTED)
                {   /*Restart scanning if time out happens*/
                    CyBle_GapcStartScan(CYBLE_SCANNING_SLOW); //_GG_
                }
            break;


        /**********************************************************
        *                       GAP Events
        ***********************************************************/

		 case CYBLE_EVT_GAP_DEVICE_CONNECTED:
            bleActive =1;
            break;

        case CYBLE_EVT_GAP_AUTH_REQ:
            printf("(GAP_AUTH_REQ)\n");
            break;

        case CYBLE_EVT_GAP_AUTH_FAILED:
            printf("GAP_AUTH_FAILED, reason: ");
            switch(*(CYBLE_GAP_AUTH_FAILED_REASON_T *)eventParam)
            {
                case CYBLE_GAP_AUTH_ERROR_CONFIRM_VALUE_NOT_MATCH:
                    printf("CONFIRM_VALUE_NOT_MATCH\r\n");
                    break;

                case CYBLE_GAP_AUTH_ERROR_INSUFFICIENT_ENCRYPTION_KEY_SIZE:
                    printf("INSUFFICIENT_ENCRYPTION_KEY_SIZE\r\n");
                    break;

                case CYBLE_GAP_AUTH_ERROR_UNSPECIFIED_REASON:
                    printf("UNSPECIFIED_REASON\r\n");
                    break;

                case CYBLE_GAP_AUTH_ERROR_AUTHENTICATION_TIMEOUT:
                    printf("AUTHENTICATION_TIMEOUT\r\n");
                    break;

                default:
                    printf("0x%x  \r\n", *(CYBLE_GAP_AUTH_FAILED_REASON_T *)eventParam);
                    break;
            }
            break;

        case CYBLE_EVT_GAPC_CONNECTION_UPDATE_COMPLETE:
            printf("(GAPC_CONN_UPDATE_COMPLETE)\n");
            break;

        case CYBLE_EVT_GAP_DEVICE_DISCONNECTED:
            printf("(GAP_DEVICE_DISCONNECTED, reason: %x)\n", *(uint8*)eventParam);
            /* Put the device into discoverable mode so that remote can search it. */
    		bleActive =0;

            if(CyBle_GetState()==CYBLE_STATE_DISCONNECTED)
            {   // _GG_ ci vuole! All'avvio si ferma!
                CyBle_GappStartAdvertisement(CYBLE_ADVERTISING_FAST);
            }
            break;

#if 0
//        case CYBLE_EVT_GAP_AUTH_COMPLETE:
//            DBG_PRINTF("CYBLE_EVT_GAP_AUTH_COMPLETE: security:%x, bonding:%x, ekeySize:%x, authErr %x \r\n",
//                        ((CYBLE_GAP_AUTH_INFO_T *)eventParam)->security,
//                        ((CYBLE_GAP_AUTH_INFO_T *)eventParam)->bonding,
//                        ((CYBLE_GAP_AUTH_INFO_T *)eventParam)->ekeySize,
//                        ((CYBLE_GAP_AUTH_INFO_T *)eventParam)->authErr);
//            break;

//        case CYBLE_EVT_GAP_ENCRYPT_CHANGE:
//            DebugPrintInt("CYBLE_EVT_GAP_ENCRYPT_CHANGE: %d \r\n", *(uint8 *)eventParam);
//            break;

        case CYBLE_EVT_GAPP_ADVERTISEMENT_START_STOP:
//            DebugPrintInt("CYBLE_EVT_GAPP_ADVERTISEMENT_START_STOP, state: %x\r\n", CyBle_GetState());
//            PrintState();

//            if(CYBLE_STATE_DISCONNECTED == CyBle_GetState())
//            {
//                /* Fast and slow advertising period complete, go to low power
//                 * mode (Hibernate mode) and wait for an external
//                 * user event to wake up the device again */
//                DebugPrint("Hibernate \r\n");
//            #if (DEBUG_UART_ENABLED == ENABLED)
//                while((UART_DEB_SpiUartGetTxBufferSize() + UART_DEB_GET_TX_FIFO_SR_VALID) != 0);
//            #endif /* (DEBUG_UART_ENABLED == ENABLED) */
//                CySysPmHibernate();
//            }
            break;

        /**********************************************************
        *                       GATT Events
        ***********************************************************/
            case CYBLE_EVT_GATTC_ERROR_RSP:
            printf("(GATTC_ERROR_RSP: opcode: ");
            switch(((CYBLE_GATTC_ERR_RSP_PARAM_T *)eventParam)->opCode)
            {
                case CYBLE_GATT_FIND_INFO_REQ:
                    printf("FIND_INFO_REQ)");
                    break;

                case CYBLE_GATT_READ_BY_TYPE_REQ:
                    printf("READ_BY_TYPE_REQ)");
                    break;

                case CYBLE_GATT_READ_BY_GROUP_REQ:
                    printf("READ_BY_GROUP_REQ)");
                    break;

                default:
                    printf("%x", ((CYBLE_GATTC_ERR_RSP_PARAM_T *)eventParam)->opCode);
                    break;
            }
            printf(",  handle: %x,  ", ((CYBLE_GATTC_ERR_RSP_PARAM_T *)eventParam)->attrHandle);
            printf("errorcode: ");
            switch(((CYBLE_GATTC_ERR_RSP_PARAM_T *)eventParam)->errorCode)
            {
                case CYBLE_GATT_ERR_ATTRIBUTE_NOT_FOUND:
                    printf("ATTRIBUTE_NOT_FOUND");
                    break;

                default:
                    printf("%x", ((CYBLE_GATTC_ERR_RSP_PARAM_T *)eventParam)->errorCode);
                    break;
            }
            printf("\r\n");
            break;

        case CYBLE_EVT_GATT_CONNECT_IND:
            printf("(GATT_CONNECT_IND)\n");
            break;

        case CYBLE_EVT_GATT_DISCONNECT_IND:
            printf("(GATT_DISCONNECT_IND)\n");
            break;

        case CYBLE_EVT_GATTS_XCNHG_MTU_REQ:
            printf("(GATTS_XCNHG_MTU_REQ)\n");
            break;

        case CYBLE_EVT_GATTS_HANDLE_VALUE_CNF:
            printf("(GATTS_HANDLE_VALUE_CNF)\n");
            break;
        case CYBLE_EVT_GATTS_READ_CHAR_VAL_ACCESS_REQ:
            /* Triggered on server side when client sends read request and when
            * characteristic has CYBLE_GATT_DB_ATTR_CHAR_VAL_RD_EVENT property set.
            * This event could be ignored by application unless it need to response
            * by error response which needs to be set in gattErrorCode field of
            * event parameter. */
            
            // _GG_ non usata!
            /*
			rdReqParam = (CYBLE_GATTS_CHAR_VAL_READ_REQ_T *) eventParam;
			if (rdReqParam->attrHandle  == 0x14)
			{
			}

          	printf("(GATTS_RD: hndle: %x)\n", ((CYBLE_GATTS_CHAR_VAL_READ_REQ_T *)eventParam)->attrHandle);
            */
            break;
#endif

		//==============================================================================================
		case CYBLE_EVT_GATTS_WRITE_REQ:
            answerRequest=1;
		case CYBLE_EVT_GATTS_WRITE_CMD_REQ:
			wrReqParam = (CYBLE_GATTS_WRITE_REQ_PARAM_T *) eventParam;

//#define  SHOW_DATA_UART1
#ifdef SHOW_DATA_UART1
			{
                uint8 tmp[32],i;
			    //
			    // DEBUG: print dati scrittura carattesistica
			    //
			    MemFill(U08PTR tmp, 0, sizeof(tmp));

				// Handle dal quale arrivano dati
			    sprintf((char *)&tmp,"WR_CH: %02X, dati:", wrReqParam->handleValPair.attrHandle);  //
				printf((const char *)tmp);

				// dati...
				for (i=0; i<wrReqParam->handleValPair.value.len ; i++ )
				{
			    	sprintf((char *)&tmp,"[%02X]", wrReqParam->handleValPair.value.val[(u08)i]);  //
					printf((char *)tmp);
				}
				printf("\n");
			}
#endif
            if(wrReqParam->handleValPair.attrHandle == BLE_GATT_WRITE_HANDLE)
            {
  				//clientReadReady =1;    // se era appeso A "0"

                /* Store BLE_GATT_READ_HANDLE Characteristic data in local array */
                errore = CyBle_GattsWriteAttributeValue(&wrReqParam->handleValPair, 0, &cyBle_connHandle, 0);
                
                if (errore) errore |=0;  // compiler warning bypass

                BufCirStrCopy((STRUCT_BUFCIR *)&comm.buffCircRx, wrReqParam->handleValPair.value.val, wrReqParam->handleValPair.value.len);

				if (answerRequest)
					CyBle_GattsWriteRsp(cyBle_connHandle);  // Send the response to the write request.
            }
            if(wrReqParam->handleValPair.attrHandle == 0x14) // Client Characteristic Configuration handler
            {
                uint8 wr = *wrReqParam->handleValPair.value.val;

                if(wr == 1)
                {
                    Uart2SendString("&&&NOTIFY_ENABLED\r\n");

                    // la scrittura su questo Handle (Client Characteristic Configuration) da parte del Client
                    // mi abilita per la trasmissione delle notifiche. Si dovrebbe verificare il carattere scritto da questa scrittura
                    // al fine di essere più selettivi: "wrReqParam->handleValPair....  ecc.. " riporta il valore scritto dal Client.
    				notifyEnabled =1; // sono stato abilitato alla Notifica!
                    errore = CyBle_GattsWriteAttributeValue(&wrReqParam->handleValPair, 0, &cyBle_connHandle,CYBLE_GATT_DB_PEER_INITIATED);
                }
            }
            break;
		//==============================================================================================
#if 0
        /**********************************************************
        *                       L2CAP Events
        ***********************************************************/

        case CYBLE_EVT_L2CAP_CONN_PARAM_UPDATE_REQ:
            printf ("(L2CAP_CONN_PARAM_UPDATE_REQ)\n");
            break;

        /**********************************************************
        *                       Other Events
        ***********************************************************/
        case CYBLE_EVT_PENDING_FLASH_WRITE:
            /* Inform application that flash write is pending. Stack internal data
            * structures are modified and require to be stored in Flash using
            * CyBle_StoreBondingData() */
            printf("(PENDING_FLASH_WRITE)\n");
            break;
#endif
      
            default:
            printf("Other event: 0x%lx \r\n", event);
            break;

    }
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
extern void ISR_UART(void);
extern void ISR_UART2(void);
//================================================================================================
void InitializeSystem(void)
//================================================================================================
{
    /* Enable global interrupts */
    CyGlobalIntEnable;

    /* Internal low power oscillator is stopped as it is not used in this project */
    CySysClkIloStop();

    /* Set the divider for ECO, ECO will be used as source when IMO is switched off to save power, to drive the HFCLK.
    [C2] Configure the divider values for the ECO so that a 3-MHz ECO divided clock can be provided to the CPU in the Sleep mode.
    The clock to the CPU is switched from the IMO to the ECO before entering the Sleep mode.
    The ECO-derived clock is available when the CPU wakes up from the Sleep mode.
    */
    CySysClkWriteEcoDiv(CY_SYS_CLK_ECO_DIV8);

    CyBle_Start(StackEventHandler);

    Timer1_Start();
    TimerIrq_StartEx(Timer2mS5);
	millisecCtr = 0;

    /* Start UART operation */
    UART_Start();

    /* Start UART2 operation */
    UART2_Start();
    /* Enable interrupt for UART */
	Uart2VarInit();

    // NON USATO!   UART_EnableInt();   // UART_EnableInt()     CyIntEnable(UART_ISR_NUMBER)
    
    UART2_EnableInt();  // UART2_EnableInt()    CyIntEnable(UART2_ISR_NUMBER)


    /* Sets interrupt handler for UART */
    UART_SetCustomInterruptHandler(*ISR_UART);
    /* Sets interrupt handler for UART */
    UART2_SetCustomInterruptHandler(*ISR_UART2);
}
//#define LOW_POWER_ENABLED

#ifdef LOW_POWER_ENABLED
int lpMode,interruptsStatus,bleSsState;
//===================================================================================
void LowPowerImplementation(void)
//===================================================================================
{
    if((UART_SpiUartGetTxBufferSize() + UART_GET_TX_FIFO_SR_VALID) == 0)
    {
       if(CyBle_GetState() != CYBLE_STATE_INITIALIZING)
       {

            /* Put BLE sub system in DeepSleep mode when it is IDLE

            [C3] In the main while loop, call the function CyBle_EnterLPM()() to put the BLESS into the DEEPSLEEP mode soon after the BLE event processing is completed.
            Note that this function will internally check the BLESS conditions if it can enter the DEEPSLEEP mode.
            Therefore, no check is required before calling this function.
            */
            lpMode = CyBle_EnterLPM(CYBLE_BLESS_DEEPSLEEP);

            /* Disable global interrupts to avoid any other tasks from interrupting this section of code*/
            interruptsStatus = CyEnterCriticalSection();

            /* Get current state of BLE sub system to check if it has successfully entered deep sleep state */
            bleSsState = CyBle_GetBleSsState();

            /* If BLE sub system has entered DEEP SLEEP, put chip into deep sleep for reducing power consumption */
            if(lpMode == CYBLE_BLESS_DEEPSLEEP)
            {
                /* And it is still there or ECO is on */
                if(bleSsState == CYBLE_BLESS_STATE_ECO_ON || bleSsState == CYBLE_BLESS_STATE_DEEPSLEEP)
                {
                   //---------------------------------------------------------
                   /* Put the chip into the DEEP SLEEP state as there are no pending tasks and BLE has also successfully entered BLE DEEP SLEEP mode */
                    if(Uart2Busy())
                    {
LED_RUN_ON;
                        CySysPmSleep();     // Put the CPU into Sleep mode and let SCB to continue sending debug data
LED_RUN_OFF;
                    }
                    else
                    {
                        CySysPmDeepSleep();
                    }
                   //---------------------------------------------------------
                }
            }
            /* BLE sub system has not entered deep sleep, wait for completion of radio operations */
            else if(bleSsState != CYBLE_BLESS_STATE_EVENT_CLOSE)
            {
                /* change HF clock source from IMO to ECO, as IMO can be stopped to save power */
                CySysClkWriteHfclkDirect(CY_SYS_CLK_HFCLK_ECO);
                /* stop IMO for reducing power consumption */
                CySysClkImoStop();
                //---------------------------------------------------------
                /* put the CPU to SLEEP */
LED_RUN_ON;
                CySysPmSleep();
LED_RUN_OFF;
                //---------------------------------------------------------
                /* starts execution after waking up, start IMO */
                CySysClkImoStart();
                /* change HF clock source back to IMO */
                CySysClkWriteHfclkDirect(CY_SYS_CLK_HFCLK_IMO);
            }
#if 0
    da EASY BLE
        else /* When BLE subsystem has been put into Sleep mode or is active */
        {
            /* And hardware doesn't finish Tx/Rx opeation - put the CPU into Sleep mode */
            if(CyBle_GetBleSsState() != CYBLE_BLESS_STATE_EVENT_CLOSE)
            {
                CySysPmSleep();
            }
        }
#endif


            /*Enable interrupts */
            CyExitCriticalSection(interruptsStatus);

        }/*end of if(CyBle_GetState() != CYBLE_STATE_INITIALIZING)*/
    }
}
#endif

enum {
	DISCONNECTED_SM,
	CONNECTED_SM,
	TRANSPARENT_UART
};
u08 mainSM;
//extern int ctr2mS5;
//===================================================================================
int main(void)
//===================================================================================
{
    InitializeSystem();

    comm.delaySend_2mS5 = 10;
    do
    {
    	// 25mS delay after reset
    } while (comm.delaySend_2mS5);

    
    Uart2SendString("&&&SHDBLE-v1.0\r\n");  // prompt

    for(bleActive =0, mainSM = DISCONNECTED_SM;;)
    {

		CyBle_ProcessEvents();

#ifdef LOW_POWER_ENABLED
    	LowPowerImplementation();
#endif
    	switch (mainSM)
    	{
    		case DISCONNECTED_SM:
				//
    		    // DISCONNECTED
				//
    		    if (bleActive)
    		    {
    		        // passaggio da Disconnesso --> Connesso
					//
					Uart2VarInit();
                    Uart2SendString("&&&CONNECT\r\n");
    		        BleRxHandlerInit();
    		        mainSM =1;
    		    }
    		    break;

    		case CONNECTED_SM:
				//
    		    // CONNECTED
				//
/*    		    if (Uart2ReceiveString("&&&TUART\r"))
    		    {
                    Uart2SendString("&&&UART\r");
                
NON E' IL CASO: FUNZIONA ANCHE COSI': basta che sia CONNESSO!                
                
    		        mainSM =TRANSPARENT_UART;
    		        break;
    		    }
    		    if (!bleActive)
    		    {
    		        // passaggio da Connesso --> Disconnesso
					//
                    Uart2SendString("&&&DISCONNECT&\r");
    		        // STOP Notification
    		        BleRxHandlerInit();
    		        mainSM =DISCONNECTED_SM;
    		        break;
    		    }
    		    break;

    		case TRANSPARENT_UART:
*/				//
    		    // "TRU"  transparent Uart
				//
    		    if (!bleActive)
    		    {
    		        // STOP Notification
                    Uart2SendString("&&&DISCONNECT\r\n");
    		        mainSM =DISCONNECTED_SM;
    		        break;
    		    }
    		    else
    		    {
/*					if (uart2.escapeCtr >=3)
            		{
						// richiesta uscita da modo trasparente
                        Uart2SendString("&&&CMD\r");
    		            mainSM =CONNECTED_SM;
					}
					else  */
					{
						//-------------------------------------------------------
						// da UART2-RX  --> BLE
						//
						if (uart2.rxMsgRdy)
            			{
							// passo stringa in arrivo da Uart2 --> BLE
							uart2.rxMsgRdy =0;
							RefreshTxCharacteristic((uint8 *)&uart2.rxBuf, uart2.rxCtr);
    		        	}
						//-------------------------------------------------------
						// da BLE --> UART2-TX
						//
			            if (BufCirDataRdy((STRUCT_BUFCIR  *)&comm.buffCircRx))
            			{
   							//Ricevuto almeno un carattere da BLE
                			//
                    		uint8 tmp = BufCirGet((STRUCT_BUFCIR *)&comm.buffCircRx);
			            	UART2_UartPutChar(tmp);
						}
					}
    		    }
    		    break;
    	};
    }

 }


/* [] END OF FILE */

